class ResourceKeywordIdentifier:
    KINDS_KEYWORDS_MAP = {
        "PersistentVolumeClaim": ["claimName"],
        "ServiceAccount": ["serviceAccountName"],
        "ClusterRole": ["rules[].resources", "rules[].resourceNames"],
        "ClusterRoleBinding": [["roleRef.name", "kind"], ["subjects[].name", "kind"], ["aggregationRule.clusterRoleSelectors.matchLabels"]],
        "Role": ["rules[].resources", "rules[].resourceNames"],
        "RoleBinding": [["roleRef.name", "kind"], ["subjects[].name", "kind"]]
    }



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from checkov.azure_pipelines.checks.registry import registry
from checkov.common.output.report import CheckType
from checkov.yaml_doc.runner import Runner as YamlRunner


if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry


class Runner(YamlRunner):
    check_type = CheckType.AZURE_PIPELINES  # noqa: CCE003  # a static attribute

    def require_external_checks(self) -> bool:
        return False

    def import_registry(self) -> BaseCheckRegistry:
        return registry

    def _parse_file(
        self, f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        if self.is_workflow_file(f):
            return super()._parse_file(f=f)

        return None

    def is_workflow_file(self, file_path: str) -> bool:
        return file_path.endswith("azure-pipelines.yml")



from __future__ import annotations

import logging
import os
from typing import Any

import requests

from checkov.common.runners.base_runner import strtobool
from checkov.common.vcs.base_vcs_dal import BaseVCSDAL


class Bitbucket(BaseVCSDAL):
    def setup_conf_dir(self) -> None:
        """
            discover parameters from execution context of checkov and determine the directory to save temporal files of vcs configuration
        """
        bitbucket_conf_dir_name = os.getenv('CKV_BITBUCKET_CONF_DIR_NAME', 'bitbucket_conf')
        self.bitbucket_conf_dir_path = os.path.join(os.getcwd(), bitbucket_conf_dir_name)
        self.bitbucket_branch_restrictions_file_path = os.path.join(self.bitbucket_conf_dir_path,
                                                                    "branch_restrictions.json")

    def discover(self) -> None:
        """
            discover parameters from execution context of checkov. usually from env variable
        """
        server_host = os.getenv('CI_SERVER_URL', "https://api.bitbucket.org/")
        self.api_url = f'{server_host}2.0'
        self.graphql_api_url = f"{server_host}api/graphql"

        self.token = os.getenv('APP_PASSWORD', '')

        self.current_repository = os.getenv('BITBUCKET_REPO_FULL_NAME', '')
        self.current_branch = os.getenv('BITBUCKET_BRANCH', '')
        self.default_branch_cache = {}
        self.username = os.getenv('BITBUCKET_USERNAME', '')

    def _request(self, endpoint: str, allowed_status_codes: list[int]) -> dict[str, Any] | None:
        if not self.token:
            return None
        url_endpoint = f"{self.api_url}/{endpoint}"
        try:
            s = requests.Session()
            s.auth = (self.username, self.token)
            request = s.get(url_endpoint)
            if request.status_code in allowed_status_codes:
                data: "dict[str, Any]" = request.json()
                if isinstance(data, dict) and 'errors' in data.keys():
                    return None
                return data
            else:
                request.raise_for_status()
        except Exception:
            logging.debug(f"Query failed to run by returning code of {url_endpoint}", exc_info=True)

        return None

    def _headers(self) -> dict[str, Any]:
        # not needed here
        pass

    def get_branch_restrictions(self) -> dict[str, Any] | None:
        if self.current_repository:
            branch_restrictions = self._request(endpoint=f"repositories/{self.current_repository}/branch-restrictions",
                                                allowed_status_codes=[200])
            return branch_restrictions
        logging.debug("Environment variable BITBUCKET_REPO_FULL_NAME was not set. Cannot fetch branch restrictions.")
        return None

    def persist_branch_restrictions(self) -> None:
        branch_restrictions = self.get_branch_restrictions()

        if branch_restrictions:
            BaseVCSDAL.persist(path=self.bitbucket_branch_restrictions_file_path, conf=branch_restrictions)

    def persist_all_confs(self) -> None:
        if strtobool(os.getenv("CKV_BITBUCKET_CONFIG_FETCH_DATA", "True")):
            self.persist_branch_restrictions()



from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class BranchProtectionRequireSignedCommits(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure GitHub branch protection rules requires signed commits"
        id = "CKV_GIT_6"
        supported_resources = ["github_branch_protection_v3", "github_branch_protection"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self) -> str:
        return "require_signed_commits"

    def get_expected_value(self) -> Any:
        return True


check = BranchProtectionRequireSignedCommits()
